module("camera", package.seeall);
require("tilemap");

camera = {}
camera.x = 0
camera.y = 0
camera.scaleX = 1
camera.scaleY = 1
camera.rotation = 0

function camera:set()
  love.graphics.push()
  love.graphics.rotate(-self.rotation)
  love.graphics.scale(1 / self.scaleX, 1 / self.scaleY)
  love.graphics.translate(-self.x, -self.y)
end

function camera:unset()
  love.graphics.pop()
end

function camera:move(dx, dy)
  self.x = self.x + (dx or 0)
  self.y = self.y + (dy or 0)
end

function camera:rotate(dr)
  self.rotation = self.rotation + dr
end

function camera:scale(sx, sy)
  sx = sx or 1
  self.scaleX = self.scaleX * sx
  self.scaleY = self.scaleY * (sy or sx)
  if self.scaleX > 1 then self.scaleX = 1 end
  if self.scaleY > 1 then self.scaleY = 1 end
end

function camera:setPosition(x, y)
  self.x = x or self.x
  self.y = y or self.y
end

function camera:setScale(sx, sy)
  self.scaleX = sx or self.scaleX
  self.scaleY = sy or self.scaleY
end

function camera:constraint(screenWidth, screenHeight)
	local mapWidth = tilemap.map_w * tilemap.tile_w;
	local mapHeight = tilemap.map_h * tilemap.tile_h;
	
	if (camera.x + screenWidth * camera.scaleX > mapWidth + tilemap.tile_w) then
		camera.x = mapWidth - screenWidth * camera.scaleX + tilemap.tile_w;
	elseif (camera.x < tilemap.tile_w) then
		camera.x = tilemap.tile_w;
	end
	
	if (camera.y + screenHeight * camera.scaleY > mapHeight + tilemap.tile_w) then
		camera.y = mapHeight - screenHeight * camera.scaleY + tilemap.tile_w;
	elseif (camera.y < tilemap.tile_w) then
		camera.y = tilemap.tile_h;
	end
	
end
