--[[
--------------------------------------------------------
|     _        _   _               _ _       _     _   |
|    / \   ___| |_| |__   ___ _ __| (_) __ _| |__ | |_ |
|   / _ \ / _ \ __| '_ \ / _ \ '__| | |/ _` | '_ \| __||
|  / ___ \  __/ |_| | | |  __/ |  | | | (_| | | | | |_ |
| /_/   \_\___|\__|_| |_|\___|_|  |_|_|\__, |_| |_|\__||
|                                      |___/           |
--------------------------------------------------------
|			Created by Marcos T.B. Soares			   |
|				@Seven Game Studio(C) 				   |
|			All Rights Reserved, yadda yadda		   |
|			bla bla bla GNU LICENSE bla bla			   |
--------------------------------------------------------
]]--

drawDebug = false;

require("sprite");
require("tilemap");
require("camera");
require("collider");
require("textbox");
require("input");
require("elevator");
require("pressurePlate");
require("door");
require("sound");
plr = require("player");

--Initializes timer
startTime = love.timer.getMicroTime( );
time = 0;

--The currently active game screen
currentScreen = "menu";

--Checks if game is paused or updating
paused = false;

--Main Initialization Method
function love.load()

	--Load Sounds
	sound.loadSounds();
	
	--Initialize Physics
	love.physics.setMeter(32)
	world = love.physics.newWorld(0, 9.81*64, true);
	world:setCallbacks(collider.beginCollide, collider.endCollide, collider.preCollide, collider.postCollide);
	
	--Initialize Map and tiles
	tilemap.loadTiles("default");
	tilemap.loadMap(2);
	tilemap.initializeTiles();
	
	--Set background
	love.graphics.setBackgroundColor(104, 136, 248)
	tilemap.loadBackground("content/background.png");
	
	--Initialize plr.player
	plr.init();
	
	--Initialize Camera
	camera.camera:scale(3/4);
	
	--Initialize Test Textbox
	textbox.addToList(10, 10, 100, 100, "Random Textbox of Instructions -- Press E to show debug info -- Press Space to Jump -- Arrow Keys to Move -- Z and X to control Camera Zoom -- Enter to Close this Textbox", 0.02)
	
	--Initialize Test Objects
	elevator.Elevator:new(64, 480, 480, 960);
	door.Door:new(224, 480, true);
	pressurePlate.PressurePlate:new(256, 480, door.getByIndex(1), true);
	sound.playMusic(1);
	
end

--Main update callback function
function love.update(dt)
	
	--Does not update game if paused
	if (paused == true) then
		return
	end
	
	--Updates Objects
	textbox.updateAll(dt);
	elevator.updateAll(dt);
	pressurePlate.updateAll(dt);
	door.updateAll(dt);
	
	--Centers camera on player
	camera.camera:setPosition(plr.player.body:getX() - 400 *  camera.camera.scaleX, plr.player.body:getY() - 300 * camera.camera.scaleY);
	
	--Updates Physics
	world:update(dt)
	
	if tilemap.exitFlag ~= nil then
		tilemap.exitToMap(tilemap.exitFlag);
	end
	
	--Updates player
	plr.update();
	
	--Camera Controls
	if love.keyboard.isDown('z') then
		camera.camera:scale(1.02);
	end
	if love.keyboard.isDown('x') then
		camera.camera:scale(0.98);
	end
	time = time + dt;
	
end

--Main Draw callback function
function love.draw()

	--Limits the camera to the current window size
	camera.camera:constraint(800, 600);
	camera.camera:set();
	
	--Pixel Shader Effect set
	--[[
	love.graphics.setPixelEffect(bloom)
	bloom:send("time",time) --Time parameter
	bloom:send("nIntensity", 0.6) --Grain Intensity
	bloom:send("sIntensity", 0.2) --Scanline Intensity
	bloom:send("sCount", 400)  --Scanline Count
	]]--
	tilemap.drawMap();
	
	love.graphics.setPixelEffect();
	
	--draws objects
	pressurePlate.drawAll();
	elevator.drawAll();
	door.drawAll();
	
	--draws player
	plr.player:draw();
	
	--Prints DEBUG Information
	if(drawDebug == true) then
		love.graphics.setColor(50, 50, 50)
		love.graphics.polygon("line",plr.player.body:getWorldPoints(plr.player.shape:getPoints()))
		love.graphics.rectangle("line", plr.player.posX, plr.player.posY, plr.player.frameWidth, plr.player.frameHeight);
	
		love.graphics.setColor(255, 0, 0)
		for i,v in ipairs(world:getBodyList()) do
			for o, b in ipairs(v:getFixtureList()) do
				love.graphics.polygon("line", v:getWorldPoints(b:getShape():getPoints()));
			end
		end
	end
	--Unsets the camera. Everything drawn after this will be drawn directly onto a screen position
	camera.camera:unset();
	textbox.drawAll();
	love.graphics.setColor(255, 255, 255)
	love.graphics.print("Camera Zoom :" .. camera.camera.scaleX, 0, 0);
	love.graphics.print("FPS: "..love.timer.getFPS(),0, 20);

end

--Checks for single non-continuous input of a specified key
function love.keypressed(key, unicode)
	
	input.setPressed(key);
	
	if key == 'p' then
		paused = (not paused)
	end
	if key == 'e' then
		drawDebug = (not drawDebug)
	end
	if key == 'q' then
		debug.debug();
	end
end

function love.keyreleased(key)
	input.setPressed(key);
end

--Executed when window changes focus
function love.focus(f)
  if not f then
    print("LOST FOCUS")
	paused = true;
  else
    print("GAINED FOCUS")
	paused = false;
  end
end

bloom = love.graphics.newPixelEffect[[
extern number time;

// noise effect intensity value (0 = no effect, 1 = full effect)
extern number nIntensity;

// scanlines effect intensity value (0 = no effect, 1 = full effect)
extern number sIntensity;

// scanlines effect count value (0 = no effect, 4096 = full effect)
extern number sCount;

vec4 effect(vec4 color, Image tex, vec2 tc, vec2 pc)
{
	vec4 cTextureScreen = Texel( tex, tc);
	
	number x = tc.x * tc.y * time * 1000.0;
	x = mod( x, 13.0 ) * mod( x, 123.0 );
	number dx = mod( x, 0.01 );
	
	vec3 cResult = cTextureScreen.rgb + cTextureScreen.rgb * clamp( 0.1 + dx * 100.0, 0.0, 1.0 );
	
	vec2 sc = vec2( sin( tc.y * sCount ), cos( tc.y * sCount ) );
	
	cResult += cTextureScreen.rgb * vec3( sc.x, sc.y, sc.x ) * sIntensity;
	
	cResult = cTextureScreen.rgb + clamp( nIntensity, 0.0,1.0 ) * ( cResult - cTextureScreen.rgb );
	
	cResult = vec3( cResult.r * 0.3 + cResult.g * 0.59 + cResult.b * 0.11 ); //grayscale
	return vec4( cResult, cTextureScreen.a );
	
	}
	]]

