--------------------------
--	  TVDi-Gameworks	--
--	Author : Marcos TBS --
--  @Seven Game Studio  --
--	  Version 1.0.1b	--
--    	 30/05/2012  	--
--     Sprite Class 	--
--------------------------

module("sprite", package.seeall);
require("tilemap");

--Main Sprite Table
Sprite = { posX = 0,
		   posY = 0,
		   speedX = 0,
		   speedY = 0,
		   currentFrame = 0,
		   numFrames = 0, 
		   lastTime = 0, 
		   animationTime = 0,
		   frameWidth = 0,
		   frameHeight = 0, 
		   quads = {},
		   currentLayer = 0,
		   numLayers = 0,
		   canvas = nil };

--Sprite metatable
Sprite_mt = { __index = Sprite };

--Method to create a new instance of the sprite object
function Sprite:new(x,y,dCanvas, width, height, frames, layers, thetime)
	local newInstance = {};
	setmetatable(newInstance, Sprite_mt);
	newInstance.posX = x;
	newInstance.posY = y;
	newInstance.canvas = dCanvas;
	newInstance.frameWidth = width;
	newInstance.frameHeight = height;
	newInstance.numFrames = frames or 0;
	newInstance.numLayers = layers or 1;
	newInstance.animationTime = thetime or 0;
	newInstance.quads = generateQuads(newInstance.frameWidth, newInstance.frameHeight, newInstance.numFrames, newInstance.numLayers);
	return newInstance;	
end

--Method to generate the quads that will be drawn
function generateQuads(width, height, frames, layers)
	local Quad = love.graphics.newQuad;
	local dQuads = {}
	for i=0,layers do
		for j=0,frames do
			dQuads[j+(frames * i)]= Quad( width * j, height * i, width, height, width * (frames + 1), height * layers)
		end
	end
	return dQuads;
end

--Implicit method to update a Sprite instance
function Sprite:update(dt)
	if (self.numFrames > 0) then
		if (love.timer.getMicroTime( ) - self.lastTime > self.animationTime) then
			self.currentFrame = self.currentFrame + 1;
			if self.currentFrame >= self.numFrames then
				self.currentFrame = 0;
			end
			self.lastTime = love.timer.getMicroTime();
		end
	end
end

--Implicit method to draw a Sprite instance
function Sprite:draw()
	love.graphics.drawq( self.canvas, 
						 self.quads[self.currentFrame + (self.currentLayer * self.numFrames)],
						 self.posX + tilemap.map_offset_x, 
						 self.posY + tilemap.map_offset_y)
end

function Sprite:checkCollision(sprite)
	if (sprite.posX > self.posX + self.frameWidth) then
		return false;
	elseif (sprite.posX + sprite.frameWidth < self.posX) then
		return false;
	elseif (sprite.posY > self.posY + self.frameHeight) then
		return false;
	elseif (sprite.posY + sprite.frameHeight < self.posY) then
		return false;
	else
		return true;
	end
end

--UNUSED
function Sprite:checkTouchingTiles()

	local cx = tilemap.map_offset_x;
	local cy = tilemap.map_offset_y;
	
	local topLeft = tilemap.getTileId(self.posX - cx, self.posY - cy);
	local topMid = tilemap.getTileId(self.posX + self.frameWidth/2 -cx, self.posY - cy);
	local topRight = tilemap.getTileId(self.posX + self.frameWidth - cx, self.posY - cy);
	local lowLeft = tilemap.getTileId(self.posX - cx, self.posY + self.frameHeight - cy);
	local lowMid = tilemap.getTileId(self.posX + self.frameWidth/2 - cx, self.posY + self.frameHeight - cy)
	local lowRight = tilemap.getTileId(self.posX + self.frameWidth - cx, self.posY + self.frameHeight - cy)
	
	return topLeft, topMid, topRight, lowLeft, lowMid, lowRight;
end