module("textbox", package.seeall);


textboxList = {};

Textbox = {width = 0,
		   height = 0,
		   posX = 0,
		   posY = 0,
		   text = "",
		   drawString = "",
		   textSpeed = 0,
		   lastTime = 0,
		   textDuration = 0,
		   slen = 0,
		   active = true
		  };

Textbox_mt = { __index = Textbox };

--[[
Generates a new Instance of the textbox given the following parameters:
Max window width and height, window position (x,y), 
duration in ms of each character's drawspeed (0 for instant text)
duration of text after finished drawing (0 for manual closing of textbox by player input)
--]]
function Textbox:new(width, height, posX, posY, text, textSpeed, textDuration)
	local newInstance = {};
	setmetatable(newInstance, Textbox_mt);
	newInstance.width = width;
	newInstance.height = height;
	newInstance.posX = posX;
	newInstance.posY = posY;
	newInstance.text = text;
	newInstance.textSpeed = textSpeed;
	newInstance.lastTime = love.timer.getMicroTime();
	newInstance.textDuration = textDuration or 0;
	return newInstance;
end


function Textbox:update(dt)
	
	if self.slen >= string.len(self.text) -1 then
		
		if self.textDuration <= 0 then
			if love.keyboard.isDown('return') then
				self.active = false;
			end
		elseif love.timer.getMicroTime() - self.lastTime > self.textDuration then
			self.active = false;
		end
		
		return;
	end
	
	if love.timer.getMicroTime() - self.lastTime > self.textSpeed then
		self.slen = self.slen + 1
		self.drawString = string.sub(self.text, 0, self.slen -string.len(self.text));
		self.lastTime = love.timer.getMicroTime()
	end
end


function Textbox:draw()
	
	if self.active == false then return end;
	
	local wrapping = "left"
	love.graphics.setColor(50, 50, 50, 175);
	local length = string.len(self.text) * 7;
	local height = math.ceil(length / 240) * 12;
	
	if length > 240 then 
		length = 240 
		wrapping = "center"
	end
	
	love.graphics.rectangle("fill", self.posX - 20, self.posY - 10, length + 40, height + 40);
	love.graphics.setColor(255, 255, 255);
	love.graphics.printf(self.drawString, self.posX, (self.posY), 240, wrapping);
end

function addToList(width, height, posX, posY, text, textSpeed, textDuration)
	local inst = Textbox:new(width, height, posX, posY, text, textSpeed, textDuration);
	table.insert(textboxList, inst);
end

function updateAll(dt)
	for i,v in ipairs(textboxList) do
		if v.active == false then
			table.remove(textboxList, i);
			return;
		end
		v:update(dt);
	end
end

function drawAll()
	for i,v in ipairs(textboxList) do
		v:draw()
	end
end