module("tilemap", package.seeall);
require("camera");

--Map array
local map = {};
	   
--Contains the solid tiles that will be initialized	   
solidTiles = {1, 4};	   
	   
--Stores bodies, shapes and fixtures of tiles
local physTiles = {};	   

--Stores tile images
local tiles = {};

local map_x = 0
local map_y = 0
local map_display_w = 25
local map_display_h = 20
local currentLevel = 2

map_w = 41
map_h = 20
map_offset_x = 0
map_offset_y = 0
tile_w = 32
tile_h = 32

exitFlag = nil;

local background = nil;

--Loads background that will be displayed on the screen
function loadBackground(file)
	background = love.graphics.newImage(file);
end

--Initializes map array
function loadMap(id)
	func = love.filesystem.load("content/level"..id..".lua")
	map_w, map_h, map = func();
end

--Loads all the tile images on the specified folder
function loadTiles(path)
	for i=1,3 do
		tiles[i] = love.graphics.newImage("content/tiles/"..path.."/"..i..".png")
	end
end

--Draws the tiles on screen
function drawMap()
	
	if(background ~= nil) then love.graphics.draw(background, camera.camera.x, camera.camera.y, 0) end;

	--Scans the map array for valid values and draws the corresponding tile on screen
	for i=1,map_w do
		for j=1,map_h do
			if (type(map[j][i]) ==  "number") then
				if (map[j][i] > 0) then
					if onCamera(i, j) == true then
						love.graphics.draw(tiles[map[j+map_y][i + map_x]], i * tile_w + map_offset_x, j * tile_h + map_offset_y);
					end
				end
			end
		end		
	end
end

--Gets the ID of the tile in the x-y coordinate of the map
function getTileId(x,y)
	local coordX = math.floor((x + map_offset_x)/tile_w);
	local coordY = math.floor((y + map_offset_y)/tile_h);
	return map[coordY][coordX];
end

--Creates Physical Objects and Object Instances of solid or interactive tiles
function initializeTiles()
	--Scans the map vertically
	for y=1,map_h do
		--and horizontally
		local x = 1;
		while x <= map_w do
			--Generate solid blocks when tile ID equals 1
			if map[y][x] == 1 then
				local n = 1;
				local i = x;
				while n == 1 do
					i = i+1;
					n = map[y][i];
				end
				
				physTiles[table.getn(physTiles)+1] = {}
				physTiles[table.getn(physTiles)].body = love.physics.newBody(world, x * tile_w + (tile_w/2) * (i-x), y * tile_h + tile_h/2)
				physTiles[table.getn(physTiles)].shape = love.physics.newRectangleShape(tile_w * (i-x), tile_h)
				physTiles[table.getn(physTiles)].fixture = love.physics.newFixture(physTiles[table.getn(physTiles)].body, physTiles[table.getn(physTiles)].shape) --attach shape to body
				physTiles[table.getn(physTiles)].fixture:setUserData("underpass");
				
				x = i + 1;
			--And level exits when ID is a string starting with L, followed by the level ID (ex. L5 or L20)
			else
				if string.find(map[y][x], "L") ~= nil then
					physTiles[table.getn(physTiles)+1] = {}
					physTiles[table.getn(physTiles)].body = love.physics.newBody(world, x * tile_w + tile_w/2, y * tile_h + tile_h/2)
					physTiles[table.getn(physTiles)].shape = love.physics.newRectangleShape(tile_w, tile_h)
					physTiles[table.getn(physTiles)].fixture = love.physics.newFixture(physTiles[table.getn(physTiles)].body, physTiles[table.getn(physTiles)].shape)
					physTiles[table.getn(physTiles)].fixture:setSensor(true);
					physTiles[table.getn(physTiles)].fixture:setUserData("ExitToLevel"..string.sub(map[y][x],2));
				end
				x = x + 1;
			end
		end
	end
end

--UNUSED
function checkCollisionTile(x,y,width,height,tile_x,tile_y)
	if (x - map_offset_x> tile_x + tile_w) then
		return false;
	elseif (x + width - map_offset_x < tile_x) then
		return false;
	elseif (y - map_offset_y > tile_y + tile_h) then
		return false;
	elseif (y + height - map_offset_y < tile_y) then
		return false;
	else
		return true;
	end
end

--Verifies if the tile on (x,y) coordinates is located on the visible part of the screen
function onCamera(x,y)
	if ((x * tile_w) - camera.camera.x > 832) or
	   ((x * tile_w) - camera.camera.x < -32) or
	   ((y * tile_h) - camera.camera.y > 632) or
	   ((y * tile_h) - camera.camera.y < -32) 
	then
		return false;
	else
		return true;
	end
end


--Switches the current level to a new one
function exitToMap(mapName)
	
	--Turns the Userdata string sent by the Box2D engine into a map value
	local mapId = tonumber(string.sub(mapName, 12));
	
	--Loads the new map array
	map = {}
	loadMap(mapId);
	
	--Scans the array for exit position and moves player to that spot.
	for x=1,map_w do
		for y=1,map_h do
			if type(map[y][x]) == "string" then
				if (string.find(map[y][x], "E"..currentLevel) == 1) then
					plr.player.body:setPosition(tile_w * x, tile_h * y);
				end
			end
		end
	end
	
	--Delete old map tiles
	for i=1,table.getn(physTiles) do
		physTiles[i].body:destroy();
	end
	physTiles = {};
	
	--Generate the new map's tiles
	initializeTiles();
	
	--Sets the current level ID to the new map
	currentLevel = mapId;
	
	--Garbage collection to free memory usage
	collectgarbage();
	
	--Sets flag to nil to avoid multiple calls of the method	 
	exitFlag = nil;
end